// Substitution (expression distribution for variables) -*- c++ -*-

#ifndef SUBSTITUTION_H_
# define SUBSTITUTION_H_
# pragma interface

# include <map>

/** @file Substitution.h
 * Map from variables (quantifiers or function parameters) to expressions
 */

/** Substitution (expression distribution for variables) */
class Substitution
{
  /** Pointer comparator */
  struct vptr
  {
    bool operator () (const class VariableDefinition* v1,
		      const class VariableDefinition* v2) const {
      return v1 < v2;
    }
  };
  /** Map from variable definitions to expressions */
  typedef std::map<const class VariableDefinition*,class Expression*,vptr> Map;
  /** Iterator to the map */
  typedef Map::iterator iterator;
  /** Constant iterator to the map */
  typedef Map::const_iterator const_iterator;

public:
  /** Constructor */
  Substitution ();
private:
  /** Copy constructor */
  Substitution (const class Substitution& old);
  /** Assignment operator */
  class Substitution& operator= (const class Substitution& old);
public:
  /** Destructor */
  ~Substitution ();

  /** Get the expression of a variable
   * @param var		identifies the variable
   * @return		the expression, or NULL if there is none
   */
  const class Expression* getExpr (const class VariableDefinition& var) const {
    const_iterator i = myExprs.find (&var);
    return i == myExprs.end () ? NULL : i->second;
  }

  /** Get the expression of a variable
   * @param var		identifies the variable
   * @return		the expression, or NULL if there is none
   */
  class Expression* getExpr (const class VariableDefinition& var);

  /** Assign an expression to a variable.
   * @param var		identifies the variable
   * @param expr	expression to be assigned
   */
  void setExpr (const class VariableDefinition& var, class Expression& expr);

private:
  /** The expressions sorted by variables */
  Map myExprs;
};

#endif // SUBSTITUTION_H_
