// List of unprocessed states -*- c++ -*-

#ifndef STATELIST_H_
# define STATELIST_H_
# pragma interface

# include <list>
# include "BitBuffer.h"

/** @file StateList.h
 * List of unprocessed states
 */

/** List of unprocessed states */
class StateList
{
  struct item
  {
    word_t* data;
    size_t size;
  };
  typedef std::list<struct item> List;
public:
  /** Constructor */
  StateList () : myList () {}
private:
  /** Copy constructor */
  StateList (const class StateList& old);
  /** Assignment operator */
  class StateList& operator= (const class StateList& old);
public:
  /** Destructor */
  ~StateList () {
    clear ();
  }

  /** Determine whether the list is empty */
  bool empty () const { return myList.empty (); }
  /** Clear the list */
  void clear () {
    for (List::iterator i = myList.begin (); i != myList.end (); i++)
      delete[] i->data;
    myList.clear ();
  }
  /** Insert an encoded state to the list
   * @param buf		the encoded state
   * @param size	length of the encoded state in bytes
   */
  void push (const void* buf,
	     size_t size) {
    size_t last = (size - 1) / sizeof (word_t);
    struct item item = { new word_t[last + 1], size };
    item.data[last] = 0;
    memcpy (item.data, buf, size);
    myList.push_front (item);
  }
  /** Fetch an encoded state from the list
   * @param size	(output) length of the encoded stream
   * @param tail	flag: retrieve from tail of list instead of head
   */
  word_t* pop (bool tail,
	       size_t* size = 0) {
    List::iterator i;
    if (tail)
      (i = myList.end ())--;
    else
      i = myList.begin ();
    struct item item = *i;
    myList.erase (i);
    if (size)
      *size = item.size;
    else
      BitPacker::inflate (item.data[(item.size - 1) / sizeof (word_t)],
			  (-item.size) % sizeof (word_t));
    return item.data;
  }

private:
  /** The list of unprocessed states */
  List myList;
};

#endif // STATELIST_H_
