// Place class -*- c++ -*-

#pragma implementation
#include "Place.h"
#include "Constraint.h"
#include "Arc.h"
#include "Marking.h"
#include "CardType.h"
#include "LeafValue.h"
#include "PlaceMarking.h"
#include "Printer.h"
#include "util.h"

/** @file Place.C
 * Place in an algebraic system net
 */

Place::Place (unsigned index, char* name,
	      class Constraint* capacity,
	      const class Type& type) :
  myIndex (index), myName (name), myCapacity (capacity),
  myCapacityBits (CARD_T_BIT), myType (&type),
  myInitMarking (0), myIsImplicit (false), myIsVisible (false),
  myIsNonempty (false), myIsSuppressed (false), myMaxNumTokens (CARD_T_MAX)
{
  if (myCapacity) {
    myCapacityBits = log2 (myCapacity->getNumValues ());
    card_t n;
    if (!CardType::convert (n, 0, *myCapacity))
      myIsNonempty = true;

    const class Value& v = myCapacity->getLastValue ();
    assert (v.getKind () == Value::vLeaf);
    myMaxNumTokens = card_t (static_cast<const class LeafValue&>(v));
  }
}

Place::~Place ()
{
  delete[] myName;
  delete myCapacity;
  myInitMarking->destroy ();
}

void
Place::setInitMarking (class Marking& initMarking)
{
  assert (!myInitMarking && !myIsImplicit);
  myInitMarking = &initMarking;
  class Valuation v;
  if (class PlaceMarking* p = myInitMarking->meval (v))
    delete p;
  else
    myIsImplicit = true;
}

void
Place::display (const class Printer& printer) const
{
  printer.printRaw ("place");
  printer.delimiter (' ');
  printer.print (myName);
  
  if (myCapacity) {
    printer.delimiter (' ');
    myCapacity->display (printer);
  }

  printer.delimiter (' ');
  if (const char* name = myType->getName ())
    printer.print (name);
  else
    myType->display (printer);

  if (myInitMarking) {
    printer.delimiter (':');
    myInitMarking->display (printer);
  }

  printer.delimiter (';');
  printer.finish ();
}
